////////////////////////////////////////////////////////////////////////////////
//                                  Include                                   //
////////////////////////////////////////////////////////////////////////////////


#include "Oracle.h"


////////////////////////////////////////////////////////////////////////////////
//                              Public: Oracle                                //
////////////////////////////////////////////////////////////////////////////////


Oracle::Oracle(char* filename) : x(env), u(env), cx(env), lag(env), obj(env),
				 model(env), cplex(env)
{
  try {
    //----- User: Create data model -------------------------------------------1

    // Initiate variables
    IloInt j;

    // m (rows), n (cols)
    m = 1;
    n = 2;

    // x (primal variables)
    x.add(IloNumVarArray(env, n, 0, IloInfinity));

    // u (dual multipliers)
    u.add(IloNumArray(env, m));

    // model: primal object
    cx = x[0] + 2 * x[1];

    // model: lagrange relaxed constraint (subgradient)
    lag = -(x[0] + 4 * x[1] - 8);

    // model: maximize
    obj = IloMaximize(env, cx + u[0] * lag);
    model.add(obj);

    // model: x constraints
    for (j = 0; j < n; j++) {
      model.add(x[j] >= 0);
      model.add(x[j] <= 4);
    }

    // cplex: extract model
    cplex.extract(model);

    //-------------------------------------------------------------------------1

    //----- User: Variables that must be set ----------------------------------2

    // u size
    uSize = m;

    // x dimensions and size
    xDims = 1;
    xDim1Size = n;

    // Add all unconstrained multipliers u, end with an InINF
    ucVector = new unsigned int[1];
    ucVector[0] = InINF;

    // Is the primal problem of min type
    isMinProblem = false;

    //-------------------------------------------------------------------------2

    iteration = 0;

  } catch(IloException& e) {
    cerr << "Error: " << e.getMessage() << endl;
  } catch(...) {
    cout << "Error" << endl;
  }
}


////////////////////////////////////////////////////////////////////////////////
//                              Public: ~Oracle                               //
////////////////////////////////////////////////////////////////////////////////


Oracle::~Oracle()
{
  env.end();
}


////////////////////////////////////////////////////////////////////////////////
//                               Public: Solve                                //
////////////////////////////////////////////////////////////////////////////////


void Oracle::Solve(const double uVector[], double &fValue, double sgVector[],
		   double xVector[])
{
  //----- User: Solve model ---------------------------------------------------1

  // Inititate variables
  IloInt i, j;

  // Update u
  u.clear();
  for (i = 0; i < m; i++)
    u.add(uVector[i]);

  // Update model
  model.remove(obj);
  obj = IloMaximize(env, cx + u[0] * lag);
  model.add(obj);

  // Solve model
  cplex.solve();

  // fValue
  fValue = cplex.getValue(obj);

  // sgVector
  sgVector[0] = cplex.getValue(lag);

  // xVector
  for (j = 0; j < n; j++)
    xVector[j] = cplex.getValue(x[j]);

  // Print iteration
  iteration += 1;
  cout << endl;
  cout << "Iteration " << iteration << endl;
  cout << "------------------------------" << endl;
  cout << "  f = " << fValue << endl;
  for (i = 0; i < m; i++)
    cout << "  u" << i + 1 << " = " << u[i] << endl;
  for (j = 0; j < n; j++)
    cout << "  x" << j + 1 << " = " << cplex.getValue(x[j]) << endl;
  for (i = 0; i < m; i++)
    cout << "  g" << i + 1 << " = " << sgVector[i] << endl;
  cout << "------------------------------" << endl;
  cout << endl;

  //---------------------------------------------------------------------------1
}


////////////////////////////////////////////////////////////////////////////////
//                                Public: getX                                //
////////////////////////////////////////////////////////////////////////////////


long Oracle::getUSize()
{
  return uSize;
}

long Oracle::getXDims()
{
  return xDims;
}

long Oracle::getXDim1Size()
{
  return xDim1Size;
}

long Oracle::getXDim2Size()
{
  return xDim2Size;
}

bool Oracle::getIsMinProblem()
{
  return isMinProblem;
}

unsigned int* Oracle::getUCVector()
{
  return ucVector;
}


////////////////////////////////////////////////////////////////////////////////
